function clr=mkfitpoly2nd(nd,degree,digits);
% mkfitpoly2nd.......fit layer polynomials to discrete velocity model#
%
% call: clr=mkfitpoly2nd(nd,degree,digits);
%
%      nd: (structure array)
%             velocity model in ND format, as returned by MKREADMODEL
%
%      degree: polynomial degree to be used for fitting
%
%      digits: precision to which the polynomial coefficients are rounded.
%              There will be DIGITS siginifcant digits,
%              example: if DIGITS is 4, then 
%                       3.141592653 will be reduced to 3.141, and
%                       27.182818 will be reduced to 27.18.
%
% result: clr: (structure)
%              CLR model structure as returned by MKREADCLR, containing a
%              polynomial fit to the discrete model given in nd.
%
%
% This routine subdivides the model ND at the given discontinuities and
% fits polynomials with the given degree, using matlab's polyfit.
%
% Martin Knapmeyer, 05.02.2008, 06.02.2008
%
% BUGS & LIMITATIONS: the routine does not yet transfer discontinuity names
%                     into the proper fields of the CLR struct.


%%% transform z into normailzed radius as used for PREM
nd.z=(nd.rp-nd.z)/nd.rp;

%%% identify all discontinuities, plus planet's center
dz=[0; nd.conr; nd.moho; nd.d410; nd.d520; nd.d660; nd.cmb; nd.icb; nd.dz'; nd.rp];

%%% transform discontinuity depths into normalized radius
dz=(nd.rp-dz)/nd.rp;

%%% remove all NaNs
dz=dz(~isnan(dz));


%%% plot velocity model for later comparisons
figure(2);
clf;
figure(1);
clf;
plot(nd.vp,nd.z,'.-');
hold on
plot(nd.vs,nd.z,'.-');
plot(nd.rho,nd.z,'.-');
plot(nd.qp,nd.z,'.-');
plot(nd.qs,nd.z,'.-');
hold off


%%% loop over discontinuities
disccnt=length(dz);
for discindy=2:disccnt
   
   
   %%%%%% interpolate Vp from the above discontinuity down to the current one
   
   %%% identify samples of this layer
   topdepth=dz(discindy-1);
   bottomdepth=dz(discindy);
   samples=find((nd.z<=topdepth)&(nd.z>=bottomdepth));
   samples=samples(2:(end-1)); % removes double depths at both ends
   x=nd.z(samples);
   y=nd.vp(samples);
   
   %%% plot selected depth samples
   figure(1);
   hold on
   plot(y,x,'ro');
   hold off
   
   %%% construct polynomial
   poly=polyfit(x,y,degree);
   
   %%% round polynomial coefficients
   predotdigits=floor(log10(abs(poly))+1); % digits left o decimal dot
   factor=10.^(digits-predotdigits);
   poly=round(poly.*factor)./factor;
   
   %%% plot polynomial
   figure(1);
   hold on
   plot(polyval(poly,x),x,'mx');
   plot(polyval(poly,min(x):0.01:max(x)),min(x):0.01:max(x),'m-');
   hold off
   
   %%% plot relative misfit
   misfit=(y-polyval(poly,x))./y;
   figure(2);
   hold on
   plot(x,misfit);
   hold off
   figure(1);
   figure(3);
   hold on
   plot3(degree,digits,max(abs(misfit)),'o-');
   hold off
   figure(1);
   
   
   %%% write polynomial to screen
  % format='%6.3f %6.3f %12.3f ';
  % disp(sprintf(format,topdepth,bottomdepth,poly));
  %disp(poly);
   
   
   %%%%%% Vp interpolation done
   
end; % for disccnt