function clr=mkfitpoly2nd(nd,maxdegree,digits,maxmisfit,dowrite);
% mkfitpoly2nd.......fit layer polynomials to discrete velocity model#
%
% call: clr=mkfitpoly2nd(nd,maxdegree,digits);
%       clr=mkfitpoly2nd(nd,maxdegree,digits,'dowrite');
%
%      nd: (structure array)
%             velocity model in ND format, as returned by MKREADMODEL
%
%      maxdegree: maximum polynomial degree to be used for fitting
%                 The program tries to find the smallest degree sufficient
%                 for a good fit
%
%      digits: precision to which the polynomial coefficients are rounded.
%              There will be DIGITS siginifcant digits,
%              example: if DIGITS is 4, then 
%                       3.141592653 will be reduced to 3.141, and
%                       27.182818 will be reduced to 27.18.
%
%      maxmisfit: a fit is acceptable if the misfit is smaller than this
%                 This is an absolute value in the units of the quantities
%                 to be fitted!
%
%      'dowrite': if present, tables of the polynomial coefficients are
%                 written to the screen.
%
% result: clr: (structure)
%              CLR model structure as returned by MKREADCLR, containing a
%              polynomial fit to the discrete model given in nd.
%
%
% This routine subdivides the model ND at the given discontinuities and
% fits polynomials with the given degree, using matlab's polyfit.
%
% Martin Knapmeyer, 05.02.2008, 06.02.2008
%
% BUGS & LIMITATIONS: does not really work. But was sufficient to construct
%                     polynomials for the one model and purpose I needed
%                     them for... MK07022008
%
%                     the routine does not yet transfer discontinuity names
%                     into the proper fields of the CLR struct.
%
%                     The routine does not recognize discontinuities that
%                     are not listed in the standard discontinuities fields
%                     or in the .dz field of the model structure.
%
%                     DOES NOT YET WRITE THE CLR STRUCTURE!





%%%%%% first step: identify discontinuities in OLDMODEL
%%%%%%             As discontinuity, we define any case in which the same depth sample
%%%%%%             appears twice. This might be the case at velocity jumps or at places
%%%%%%             where v(z) has a kink and the first derivative jumps.
%%%%%%             This is not done by MKREADND, so we have to do it here.
%%%%%%             The identification is simple: whenever the difference between
%%%%%%             adjacent depth samples is zero, we have a discontinuity.
%%%%%%             we assume that all depth samples are ordered by depth
%%%%%%             properly!
z=nd.z;
deltaz=diff(z);
disconindies=find(deltaz==0); % array index to discontinuities: points to the upper sample
disconcnt=length(disconindies); % number of discontinuities
discondepths=z(disconindies); % depths of discontinuities
disp(['MKFITPOLY2ND: detected layers: ' int2str(length(discondepths)+2)]);

%%% add surface and planet center to discontinuity list
dz=[0; discondepths; nd.rp];

%%% identify all discontinuities, plus planet's center
%dz=[0; nd.conr; nd.moho; nd.d410; nd.d520; nd.d660; nd.cmb; nd.icb; nd.dz'; nd.rp];

%%% transform z into normailzed radius as used for PREM
nd.z=(nd.rp-nd.z)/nd.rp;

%%% transform discontinuity depths into normalized radius
dzsav=dz; % save for later use - without rounding problems
dz=(nd.rp-dz)/nd.rp;

%%% remove all NaNs
dzsav=dzsav(~isnan(dzsav));
dz=dz(~isnan(dz));


%%% plot velocity model for later comparisons
% figure(2);
% clf;
figure(1);
clf;
plot(nd.vp,nd.z,'b.-');
hold on
plot(nd.vs,nd.z,'r.-');
plot(nd.rho,nd.z,'m.-');
plot(nd.qp,nd.z,'k.-');
plot(nd.qs,nd.z,'g.-');
hold off


warning off


%%% loop over discontinuities
disccnt=length(dz);
for discindy=2:disccnt
   
   %%% identify sample indices of this layer
   topdepth=dz(discindy-1);
   bottomdepth=dz(discindy);
   samples=find((nd.z<=topdepth)&(nd.z>=bottomdepth));
   samples=samples(2:(end-1)); % removes double depths at both ends
   x=nd.z(samples);
   
   %%%%%% interpolate Vp from the above discontinuity down to the current one
   
         quantity=' Vp'; % string used in screen output
   
         %%% identify samples of this layer
         y=nd.vp(samples);
         
         %%% mark layer in plot
         hold on
         plot([min(y) max(y)],[1 1]*topdepth,'k-');
         plot([min(y) max(y)],[1 1]*bottomdepth,'k-');
         plot(y,x,'bo');
         hold off
   
         %%% loop over polynomial degree, to find the smalles sufficient degree
         degree=0;
         done=0;
         while done==0
            
            if degree<length(y)

               %%% construct polynomial
%                disp(['layer ' int2str(discindy),...
%                      ': testing degree ' int2str(degree),...
%                      ' (' int2str(length(y)) ' points)']);
               poly=polyfit(x,y,degree);

               %%% round polynomial coefficients
               %%% if a coeff is exactly 0, then the determination of the number
               %%% of digits left of the dot yields inf. These must be replaced
               %%% by something more meaningful.
               predotdigits=zeros(size(poly))+1; 
               nonzeros=find(poly~=0);
               predotdigits(nonzeros)=floor(log10(abs(poly(nonzeros)))+1); % digits left o decimal dot
               factor=10.^(digits-predotdigits);
               poly=round(poly.*factor)./factor;

               %%% determine misfit
               %%% The misfit is the maximum deviation between the given samples and
               %%% the rounded polynomial approximation of their values.
               %%% this is a measure in the units of the interpolated quantity,
               %%% not a relative measure!!
               misfit=abs(max((y-polyval(poly,x))));
%                disp(['layer ' int2str(discindy) ': ',...
%                      ' degree: ' int2str(degree),...
%                      ' misfit: ' num2str(misfit)]);

               %%% is the fit good enough? otherwise increase degree!
               if misfit<maxmisfit
                  done=1;
                  disp([quantity ' layer ' int2str(discindy) ': ',...
                        ' sufficient degree: ' int2str(degree),...
                        ' misfit: ' num2str(misfit,'%10.8f')]);
               else
                  if degree==maxdegree
                     %%% degree exhaust!
                     done=1;
                     disp([quantity ' layer ' int2str(discindy) ': ',...
                           ' INSUFFICIENT degree: ' int2str(degree),...
                           ' misfit: ' num2str(misfit)]);
                  end; % if degree==maxdegree
                  degree=degree+1;
               end; % if misfit<maxmisfit
           
            else
               done=1;
               disp([quantity ' layer ' int2str(discindy) ': ',...
                     'not enough data points for degree>' int2str(degree)]);
            end; % if degree<length(y)
         
         end; % while done

         %%% plot polynomial
         figure(1);
         hold on
         plot(polyval(poly,x),x,'ko-');
         hold off

         %%% write polynomial to screen, if requested
         if nargin==5
                   
            %%% table requested
            %%% construct format string for screen output
            %%% begn with format for the depths
            format='%8.3f  %8.3f'; % this is for the depth entries
            %%% append a coefficient format for each coeff, depending on degree
            %%% and also depending on number of significant digits
            coeff=['%' int2str(2*digits+2) '.' int2str(digits) 'f'];
            for indy=1:(degree+1)
                format=[format ' ' coeff];
            end; % for indy
            %%% finally, append a format for the output of he misfit measure
            format=[format ' (+/- %8.6f)'];
           
            %%% re-read topdepth and bottomdepth from saved list - do output in km!
            topdepth=dzsav(discindy-1);
            bottomdepth=dzsav(discindy);
            disp([quantity ': ' sprintf(format,topdepth,bottomdepth,poly,misfit)]);
            
         end; % if nargin
   
   %%%%%% Vp interpolation done
   
   
%    %%%%%% interpolate Vs from the above discontinuity down to the current one
%    
%          quantity=' Vs'; % string used in screen output
%    
%          %%% identify samples of this layer
%          y=nd.vs(samples);
%          
%          %%% mark layer in plot
%          hold on
%          plot([min(y) max(y)],[1 1]*topdepth,'k-');
%          plot([min(y) max(y)],[1 1]*bottomdepth,'k-');
%          plot(y,x,'bo');
%          hold off
%    
%          %%% loop over polynomial degree, to find the smalles sufficient degree
%          degree=0;
%          done=0;
%          while done==0
%             
%             if degree<length(y)
% 
%                %%% construct polynomial
% %                disp(['layer ' int2str(discindy),...
% %                      ': testing degree ' int2str(degree),...
% %                      ' (' int2str(length(y)) ' points)']);
%                poly=polyfit(x,y,degree);
% 
%                %%% round polynomial coefficients
%                %%% if a coeff is exactly 0, then the determination of the number
%                %%% of digits left of the dot yields inf. These must be replaced
%                %%% by something more meaningful.
%                predotdigits=zeros(size(poly))+1; 
%                nonzeros=find(poly~=0);
%                predotdigits(nonzeros)=floor(log10(abs(poly(nonzeros)))+1); % digits left o decimal dot
%                factor=10.^(digits-predotdigits);
%                poly=round(poly.*factor)./factor;
% 
%                %%% determine misfit
%                %%% The misfit is the maximum deviation between the given samples and
%                %%% the rounded polynomial approximation of their values.
%                %%% this is a measure in the units of the interpolated quantity,
%                %%% not a relative measure!!
%                misfit=abs(max((y-polyval(poly,x))));
% %                disp(['layer ' int2str(discindy) ': ',...
% %                      ' degree: ' int2str(degree),...
% %                      ' misfit: ' num2str(misfit)]);
% 
%                %%% is the fit good enough? otherwise increase degree!
%                if misfit<maxmisfit
%                   done=1;
%                   disp([quantity ' layer ' int2str(discindy) ': ',...
%                         ' sufficient degree: ' int2str(degree),...
%                         ' misfit: ' num2str(misfit,'%10.8f')]);
%                else
%                   if degree==maxdegree
%                      %%% degree exhaust!
%                      done=1;
%                      disp([quantity ' layer ' int2str(discindy) ': ',...
%                            ' INSUFFICIENT degree: ' int2str(degree),...
%                            ' misfit: ' num2str(misfit)]);
%                   end; % if degree==maxdegree
%                   degree=degree+1;
%                end; % if misfit<maxmisfit
%            
%             else
%                done=1;
%                disp([quantity ' layer ' int2str(discindy) ': ',...
%                      'not enough data points for degree>' int2str(degree)]);
%             end; % if degree<length(y)
%          
%          end; % while done
% 
%          %%% plot polynomial
%          figure(1);
%          hold on
%          plot(polyval(poly,x),x,'ko-');
%          hold off
% 
%          %%% write polynomial to screen, if requested
%          if nargin==5
%                    
%             %%% table requested
%             %%% construct format string for screen output
%             %%% begn with format for the depths
%             format='%8.3f  %8.3f'; % this is for the depth entries
%             %%% append a coefficient format for each coeff, depending on degree
%             %%% and also depending on number of significant digits
%             coeff=['%' int2str(2*digits+2) '.' int2str(digits) 'f'];
%             for indy=1:(degree+1)
%                 format=[format ' ' coeff];
%             end; % for indy
%             %%% finally, append a format for the output of he misfit measure
%             format=[format ' (+/- %8.6f)'];
%            
%             %%% re-read topdepth and bottomdepth from saved list - do output in km!
%             topdepth=dzsav(discindy-1);
%             bottomdepth=dzsav(discindy);
%             disp([quantity ': ' sprintf(format,topdepth,bottomdepth,poly,misfit)]);
%             
%          end; % if nargin
%    
%    %%%%%% Vs interpolation done
% 
%    
%    
%       %%%%%% interpolate Rho from the above discontinuity down to the current one
%    
%          quantity='Rho'; % string used in screen output
%    
%          %%% identify samples of this layer
%          y=nd.rho(samples);
%          
%          %%% mark layer in plot
%          hold on
%          plot([min(y) max(y)],[1 1]*topdepth,'k-');
%          plot([min(y) max(y)],[1 1]*bottomdepth,'k-');
%          plot(y,x,'bo');
%          hold off
%    
%          %%% loop over polynomial degree, to find the smalles sufficient degree
%          degree=0;
%          done=0;
%          while done==0
%             
%             if degree<length(y)
% 
%                %%% construct polynomial
% %                disp(['layer ' int2str(discindy),...
% %                      ': testing degree ' int2str(degree),...
% %                      ' (' int2str(length(y)) ' points)']);
%                poly=polyfit(x,y,degree);
% 
%                %%% round polynomial coefficients
%                %%% if a coeff is exactly 0, then the determination of the number
%                %%% of digits left of the dot yields inf. These must be replaced
%                %%% by something more meaningful.
%                predotdigits=zeros(size(poly))+1; 
%                nonzeros=find(poly~=0);
%                predotdigits(nonzeros)=floor(log10(abs(poly(nonzeros)))+1); % digits left o decimal dot
%                factor=10.^(digits-predotdigits);
%                poly=round(poly.*factor)./factor;
% 
%                %%% determine misfit
%                %%% The misfit is the maximum deviation between the given samples and
%                %%% the rounded polynomial approximation of their values.
%                %%% this is a measure in the units of the interpolated quantity,
%                %%% not a relative measure!!
%                misfit=abs(max((y-polyval(poly,x))));
% %                disp(['layer ' int2str(discindy) ': ',...
% %                      ' degree: ' int2str(degree),...
% %                      ' misfit: ' num2str(misfit)]);
% 
%                %%% is the fit good enough? otherwise increase degree!
%                if misfit<maxmisfit
%                   done=1;
%                   disp([quantity ' layer ' int2str(discindy) ': ',...
%                         ' sufficient degree: ' int2str(degree),...
%                         ' misfit: ' num2str(misfit,'%10.8f')]);
%                else
%                   if degree==maxdegree
%                      %%% degree exhaust!
%                      done=1;
%                      disp([quantity ' layer ' int2str(discindy) ': ',...
%                            ' INSUFFICIENT degree: ' int2str(degree),...
%                            ' misfit: ' num2str(misfit)]);
%                   end; % if degree==maxdegree
%                   degree=degree+1;
%                end; % if misfit<maxmisfit
%            
%             else
%                done=1;
%                disp([quantity ' layer ' int2str(discindy) ': ',...
%                      'not enough data points for degree>' int2str(degree)]);
%             end; % if degree<length(y)
%          
%          end; % while done
% 
%          %%% plot polynomial
%          figure(1);
%          hold on
%          plot(polyval(poly,x),x,'ko-');
%          hold off
% 
%          %%% write polynomial to screen, if requested
%          if nargin==5
%                    
%             %%% table requested
%             %%% construct format string for screen output
%             %%% begn with format for the depths
%             format='%8.3f  %8.3f'; % this is for the depth entries
%             %%% append a coefficient format for each coeff, depending on degree
%             %%% and also depending on number of significant digits
%             coeff=['%' int2str(2*digits+2) '.' int2str(digits) 'f'];
%             for indy=1:(degree+1)
%                 format=[format ' ' coeff];
%             end; % for indy
%             %%% finally, append a format for the output of he misfit measure
%             format=[format ' (+/- %8.6f)'];
%            
%             %%% re-read topdepth and bottomdepth from saved list - do output in km!
%             topdepth=dzsav(discindy-1);
%             bottomdepth=dzsav(discindy);
%             disp([quantity ': ' sprintf(format,topdepth,bottomdepth,poly,misfit)]);
%             
%          end; % if nargin
%    
%    %%%%%% rho interpolation done

end; % for disccnt


warning on